# Stubs for pyspark.sql.readwriter (Python 3.5)
#

from typing import overload
from typing import Any, Dict, List, Optional, Tuple, Union

from pyspark.sql._typing import ReadWriterOptionType
from pyspark.sql.dataframe import DataFrame
from pyspark.rdd import RDD
from pyspark.sql.context import SQLContext
from pyspark.sql.types import *

PathOrPaths = Union[str, List[str]]
TupleOrListOfString = Union[List[str], Tuple[str, ...]]

class OptionUtils: ...

class DataFrameReader(OptionUtils):
    def __init__(self, spark: SQLContext) -> None: ...
    def format(self, source: str) -> DataFrameReader: ...
    def schema(self, schema: Union[StructType, str]) -> DataFrameReader: ...
    def option(self, key: str, value: ReadWriterOptionType) -> DataFrameReader: ...
    def options(self, **options: ReadWriterOptionType) -> DataFrameReader: ...
    def load(
        self,
        path: Optional[PathOrPaths] = ...,
        format: Optional[str] = ...,
        schema: Optional[StructType] = ...,
        **options: ReadWriterOptionType
    ) -> DataFrame: ...
    def json(
        self,
        path: Union[str, List[str], RDD[str]],
        schema: Optional[StructType] = ...,
        primitivesAsString: Optional[Union[bool, str]] = ...,
        prefersDecimal: Optional[Union[bool, str]] = ...,
        allowComments: Optional[Union[bool, str]] = ...,
        allowUnquotedFieldNames: Optional[Union[bool, str]] = ...,
        allowSingleQuotes: Optional[Union[bool, str]] = ...,
        allowNumericLeadingZero: Optional[Union[bool, str]] = ...,
        allowBackslashEscapingAnyCharacter: Optional[Union[bool, str]] = ...,
        mode: Optional[str] = ...,
        columnNameOfCorruptRecord: Optional[str] = ...,
        dateFormat: Optional[str] = ...,
        timestampFormat: Optional[str] = ...,
        multiLine: Optional[Union[bool, str]] = ...,
        allowUnquotedControlChars: Optional[Union[bool, str]] = ...,
        lineSep: Optional[str] = ...,
        samplingRatio: Optional[Union[float, str]] = ...,
        dropFieldIfAllNull: Optional[Union[bool, str]] = ...,
        encoding: Optional[str] = ...,
        locale: Optional[str] = ...,
        recursiveFileLookup: Optional[bool] = ...,
    ) -> DataFrame: ...
    def table(self, tableName: str) -> DataFrame: ...
    def parquet(self, *paths: str, **options: ReadWriterOptionType) -> DataFrame: ...
    def text(
        self,
        paths: PathOrPaths,
        wholetext: bool = ...,
        lineSep: Optional[str] = ...,
        recursiveFileLookup: Optional[bool] = ...,
    ) -> DataFrame: ...
    def csv(
        self,
        path: PathOrPaths,
        schema: Optional[StructType] = ...,
        sep: Optional[str] = ...,
        encoding: Optional[str] = ...,
        quote: Optional[str] = ...,
        escape: Optional[str] = ...,
        comment: Optional[str] = ...,
        header: Optional[Union[bool, str]] = ...,
        inferSchema: Optional[Union[bool, str]] = ...,
        ignoreLeadingWhiteSpace: Optional[Union[bool, str]] = ...,
        ignoreTrailingWhiteSpace: Optional[Union[bool, str]] = ...,
        nullValue: Optional[str] = ...,
        nanValue: Optional[str] = ...,
        positiveInf: Optional[str] = ...,
        negativeInf: Optional[str] = ...,
        dateFormat: Optional[str] = ...,
        timestampFormat: Optional[str] = ...,
        maxColumns: Optional[int] = ...,
        maxCharsPerColumn: Optional[int] = ...,
        maxMalformedLogPerPartition: Optional[int] = ...,
        mode: Optional[str] = ...,
        columnNameOfCorruptRecord: Optional[str] = ...,
        multiLine: Optional[Union[bool, str]] = ...,
        charToEscapeQuoteEscaping: Optional[str] = ...,
        samplingRatio: Optional[Union[float, str]] = ...,
        enforceSchema: Optional[Union[bool, str]] = ...,
        emptyValue: Optional[str] = ...,
        locale: Optional[str] = ...,
        lineSep: Optional[str] = ...,
    ) -> DataFrame: ...
    def orc(
        self,
        path: PathOrPaths,
        mergeSchema: Optional[bool] = ...,
        recursiveFileLookup: Optional[bool] = ...,
    ) -> DataFrame: ...
    @overload
    def jdbc(
        self, url: str, table: str, *, properties: Optional[Dict[str, str]] = ...
    ) -> DataFrame: ...
    @overload
    def jdbc(
        self,
        url: str,
        table: str,
        column: str,
        lowerBound: int,
        upperBound: int,
        numPartitions: int,
        *,
        properties: Optional[Dict[str, str]] = ...
    ) -> DataFrame: ...
    @overload
    def jdbc(
        self,
        url: str,
        table: str,
        *,
        predicates: List[str],
        properties: Optional[Dict[str, str]] = ...
    ) -> DataFrame: ...

class DataFrameWriter(OptionUtils):
    def __init__(self, df: DataFrame) -> None: ...
    def mode(self, saveMode: str) -> DataFrameWriter: ...
    def format(self, source: str) -> DataFrameWriter: ...
    def option(self, key: str, value: ReadWriterOptionType) -> DataFrameWriter: ...
    def options(self, **options: ReadWriterOptionType) -> DataFrameWriter: ...
    @overload
    def partitionBy(self, *cols: str) -> DataFrameWriter: ...
    @overload
    def partitionBy(self, __cols: List[str]) -> DataFrameWriter: ...
    @overload
    def bucketBy(self, numBuckets: int, col: str, *cols: str) -> DataFrameWriter: ...
    @overload
    def bucketBy(
        self, numBuckets: int, col: TupleOrListOfString
    ) -> DataFrameWriter: ...
    @overload
    def sortBy(self, col: str, *cols: str) -> DataFrameWriter: ...
    @overload
    def sortBy(self, col: TupleOrListOfString) -> DataFrameWriter: ...
    def save(
        self,
        path: Optional[str] = ...,
        format: Optional[str] = ...,
        mode: Optional[str] = ...,
        partitionBy: Optional[List[str]] = ...,
        **options: ReadWriterOptionType
    ) -> None: ...
    def insertInto(self, tableName: str, overwrite: Optional[bool] = ...) -> None: ...
    def saveAsTable(
        self,
        name: str,
        format: Optional[str] = ...,
        mode: Optional[str] = ...,
        partitionBy: Optional[List[str]] = ...,
        **options: ReadWriterOptionType
    ) -> None: ...
    def json(
        self,
        path: str,
        mode: Optional[str] = ...,
        compression: Optional[str] = ...,
        dateFormat: Optional[str] = ...,
        timestampFormat: Optional[str] = ...,
        lineSep: Optional[str] = ...,
        encoding: Optional[str] = ...,
        ignoreNullFields: Optional[bool] = ...,
    ) -> None: ...
    def parquet(
        self,
        path: str,
        mode: Optional[str] = ...,
        partitionBy: Optional[List[str]] = ...,
        compression: Optional[str] = ...,
    ) -> None: ...
    def text(
        self, path: str, compression: Optional[str] = ..., lineSep: Optional[str] = ...
    ) -> None: ...
    def csv(
        self,
        path: str,
        mode: Optional[str] = ...,
        compression: Optional[str] = ...,
        sep: Optional[str] = ...,
        quote: Optional[str] = ...,
        escape: Optional[str] = ...,
        header: Optional[Union[bool, str]] = ...,
        nullValue: Optional[str] = ...,
        escapeQuotes: Optional[Union[bool, str]] = ...,
        quoteAll: Optional[Union[bool, str]] = ...,
        dateFormat: Optional[str] = ...,
        timestampFormat: Optional[str] = ...,
        ignoreLeadingWhiteSpace: Optional[Union[bool, str]] = ...,
        ignoreTrailingWhiteSpace: Optional[Union[bool, str]] = ...,
        charToEscapeQuoteEscaping: Optional[str] = ...,
        encoding: Optional[str] = ...,
        emptyValue: Optional[str] = ...,
        lineSep: Optional[str] = ...,
    ) -> None: ...
    def orc(
        self,
        path: str,
        mode: Optional[str] = ...,
        partitionBy: Optional[List[str]] = ...,
        compression: Optional[str] = ...,
    ) -> None: ...
    def jdbc(
        self,
        url: str,
        table: str,
        mode: Optional[str] = ...,
        properties: Optional[Dict[str, str]] = ...,
    ) -> None: ...



# Stubs for pyspark.streaming (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from pyspark.streaming.context import StreamingContext as StreamingContext
from pyspark.streaming.dstream import DStream as DStream
from pyspark.streaming.listener import StreamingListener as StreamingListener



# Stubs for pyspark.mllib.stat._statistics (Python 3.5)
#

from typing import List, Optional, overload, Union
from typing_extensions import Literal

from numpy import ndarray  # type: ignore

from pyspark.mllib.common import JavaModelWrapper
from pyspark.mllib.linalg import Vector, Matrix
from pyspark.mllib.regression import LabeledPoint
from pyspark.mllib.stat.test import ChiSqTestResult, KolmogorovSmirnovTestResult
from pyspark.rdd import RDD

CorrelationMethod = Union[Literal["spearman"], Literal["pearson"]]

class MultivariateStatisticalSummary(JavaModelWrapper):
    def mean(self) -> ndarray: ...
    def variance(self) -> ndarray: ...
    def count(self) -> int: ...
    def numNonzeros(self) -> ndarray: ...
    def max(self) -> ndarray: ...
    def min(self) -> ndarray: ...
    def normL1(self) -> ndarray: ...
    def normL2(self) -> ndarray: ...

class Statistics:
    @staticmethod
    def colStats(rdd: RDD[Vector]) -> MultivariateStatisticalSummary: ...
    @overload
    @staticmethod
    def corr(
        x: RDD[Vector], *, method: Optional[CorrelationMethod] = ...
    ) -> Matrix: ...
    @overload
    @staticmethod
    def corr(
        x: RDD[float], y: RDD[float], method: Optional[CorrelationMethod] = ...
    ) -> float: ...
    @overload
    @staticmethod
    def chiSqTest(observed: Matrix) -> ChiSqTestResult: ...
    @overload
    @staticmethod
    def chiSqTest(
        observed: Vector, expected: Optional[Vector] = ...
    ) -> ChiSqTestResult: ...
    @overload
    @staticmethod
    def chiSqTest(observed: RDD[LabeledPoint]) -> List[ChiSqTestResult]: ...
    @staticmethod
    def kolmogorovSmirnovTest(
        data, distName: Literal["norm"] = ..., *params: float
    ) -> KolmogorovSmirnovTestResult: ...



# Stubs for pyspark.streaming.listener (Python 3.7)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class StreamingListener:
    def __init__(self) -> None: ...
    def onStreamingStarted(self, streamingStarted: Any) -> None: ...
    def onReceiverStarted(self, receiverStarted: Any) -> None: ...
    def onReceiverError(self, receiverError: Any) -> None: ...
    def onReceiverStopped(self, receiverStopped: Any) -> None: ...
    def onBatchSubmitted(self, batchSubmitted: Any) -> None: ...
    def onBatchStarted(self, batchStarted: Any) -> None: ...
    def onBatchCompleted(self, batchCompleted: Any) -> None: ...
    def onOutputOperationStarted(self, outputOperationStarted: Any) -> None: ...
    def onOutputOperationCompleted(self, outputOperationCompleted: Any) -> None: ...
    class Java:
        implements: Any = ...



# Stubs for pyspark.storagelevel (Python 3.5)
#

from typing import Any

class StorageLevel:
    useDisk: bool
    useMemory: bool
    useOffHeap: bool
    deserialized: bool
    replication: int
    def __init__(
        self,
        useDisk: bool,
        useMemory: bool,
        useOffHeap: bool,
        deserialized: bool,
        replication: int = ...,
    ) -> None: ...



# Stubs for pyspark.sql.udf (Python 3)
#

from typing import Any, Callable, Optional

from pyspark.sql._typing import ColumnOrName, DataTypeOrString
from pyspark.sql.column import Column
import pyspark.sql.session

class UserDefinedFunction:
    func: Callable[..., Any]
    evalType: int
    deterministic: bool
    def __init__(
        self,
        func: Callable[..., Any],
        returnType: DataTypeOrString = ...,
        name: Optional[str] = ...,
        evalType: int = ...,
        deterministic: bool = ...,
    ) -> None: ...
    @property
    def returnType(self): ...
    def __call__(self, *cols: ColumnOrName) -> Column: ...
    def asNondeterministic(self) -> UserDefinedFunction: ...

class UDFRegistration:
    sparkSession: pyspark.sql.session.SparkSession
    def __init__(self, sparkSession: pyspark.sql.session.SparkSession) -> None: ...
    def register(
        self,
        name: str,
        f: Callable[..., Any],
        returnType: Optional[DataTypeOrString] = ...,
    ): ...
    def registerJavaFunction(
        self,
        name: str,
        javaClassName: str,
        returnType: Optional[DataTypeOrString] = ...,
    ) -> None: ...
    def registerJavaUDAF(self, name: str, javaClassName: str) -> None: ...



# Stubs for pyspark.sql (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from pyspark.sql.types import Row as Row
from pyspark.sql.context import (
    SQLContext as SQLContext,
    UDFRegistration as UDFRegistration,
)
from pyspark.sql.session import SparkSession as SparkSession
from pyspark.sql.column import Column as Column
from pyspark.sql.dataframe import (
    DataFrame as DataFrame,
    DataFrameNaFunctions as DataFrameNaFunctions,
    DataFrameStatFunctions as DataFrameStatFunctions,
)
from pyspark.sql.group import GroupedData as GroupedData
from pyspark.sql.readwriter import (
    DataFrameReader as DataFrameReader,
    DataFrameWriter as DataFrameWriter,
)
from pyspark.sql.window import Window as Window, WindowSpec as WindowSpec
from pyspark.sql.pandas.group_ops import PandasCogroupedOps



# Stubs for pyspark.ml.tuning (Python 3)

from typing import overload
from typing import Any, Dict, List, Optional, Tuple, Type
from pyspark.ml._typing import P, ParamMap

from pyspark.ml import Estimator, Model
from pyspark.ml.evaluation import Evaluator
from pyspark.ml.param import Param
from pyspark.ml.param.shared import *
from pyspark.ml.util import *

class ParamGridBuilder:
    def __init__(self) -> None: ...
    def addGrid(self, param: Param, values: List[Any]) -> ParamGridBuilder: ...
    @overload
    def baseOn(self, __args: ParamMap) -> ParamGridBuilder: ...
    @overload
    def baseOn(self, *args: Tuple[Param, Any]) -> ParamGridBuilder: ...
    def build(self) -> List[ParamMap]: ...

class _ValidatorParams(HasSeed):
    estimator: Param[Estimator]
    estimatorParamMaps: Param[List[ParamMap]]
    evaluator: Param[Evaluator]
    def getEstimator(self) -> Estimator: ...
    def getEstimatorParamMaps(self) -> List[ParamMap]: ...
    def getEvaluator(self) -> Evaluator: ...

class _CrossValidatorParams(_ValidatorParams):
    numFolds: Param[int]
    def getNumFolds(self) -> int: ...

class CrossValidator(
    Estimator[CrossValidatorModel],
    _CrossValidatorParams,
    HasParallelism,
    HasCollectSubModels,
    MLReadable[CrossValidator],
    MLWritable,
):
    def __init__(
        self,
        *,
        estimator: Optional[Estimator] = ...,
        estimatorParamMaps: Optional[List[ParamMap]] = ...,
        evaluator: Optional[Evaluator] = ...,
        numFolds: int = ...,
        seed: Optional[int] = ...,
        parallelism: int = ...,
        collectSubModels: bool = ...
    ) -> None: ...
    def setParams(
        self,
        *,
        estimator: Optional[Estimator] = ...,
        estimatorParamMaps: Optional[List[ParamMap]] = ...,
        evaluator: Optional[Evaluator] = ...,
        numFolds: int = ...,
        seed: Optional[int] = ...,
        parallelism: int = ...,
        collectSubModels: bool = ...
    ) -> CrossValidator: ...
    def setEstimator(self, value: Estimator) -> CrossValidator: ...
    def setEstimatorParamMaps(self, value: List[ParamMap]) -> CrossValidator: ...
    def setEvaluator(self, value: Evaluator) -> CrossValidator: ...
    def setNumFolds(self, value: int) -> CrossValidator: ...
    def setSeed(self, value: int) -> CrossValidator: ...
    def setParallelism(self, value: int) -> CrossValidator: ...
    def setCollectSubModels(self, value: bool) -> CrossValidator: ...
    def copy(self, extra: Optional[ParamMap] = ...) -> CrossValidator: ...
    def write(self) -> MLWriter: ...
    @classmethod
    def read(cls: Type[CrossValidator]) -> MLReader: ...

class CrossValidatorModel(
    Model, _CrossValidatorParams, MLReadable[CrossValidatorModel], MLWritable
):
    bestModel: Model
    avgMetrics: List[float]
    subModels: List[List[Model]]
    def __init__(
        self,
        bestModel: Model,
        avgMetrics: List[float] = ...,
        subModels: Optional[List[List[Model]]] = ...,
    ) -> None: ...
    def copy(self, extra: Optional[ParamMap] = ...) -> CrossValidatorModel: ...
    def write(self) -> MLWriter: ...
    @classmethod
    def read(cls: Type[CrossValidatorModel]) -> MLReader: ...

class _TrainValidationSplitParams(_ValidatorParams):
    trainRatio: Param[float]
    def getTrainRatio(self) -> float: ...

class TrainValidationSplit(
    Estimator[TrainValidationSplitModel],
    _TrainValidationSplitParams,
    HasParallelism,
    HasCollectSubModels,
    MLReadable[TrainValidationSplit],
    MLWritable,
):
    def __init__(
        self,
        *,
        estimator: Optional[Estimator] = ...,
        estimatorParamMaps: Optional[List[ParamMap]] = ...,
        evaluator: Optional[Evaluator] = ...,
        trainRatio: float = ...,
        parallelism: int = ...,
        collectSubModels: bool = ...,
        seed: Optional[int] = ...
    ) -> None: ...
    def setParams(
        self,
        *,
        estimator: Optional[Estimator] = ...,
        estimatorParamMaps: Optional[List[ParamMap]] = ...,
        evaluator: Optional[Evaluator] = ...,
        trainRatio: float = ...,
        parallelism: int = ...,
        collectSubModels: bool = ...,
        seed: Optional[int] = ...
    ) -> TrainValidationSplit: ...
    def setEstimator(self, value: Estimator) -> TrainValidationSplit: ...
    def setEstimatorParamMaps(self, value: List[ParamMap]) -> TrainValidationSplit: ...
    def setEvaluator(self, value: Evaluator) -> TrainValidationSplit: ...
    def setTrainRatio(self, value: float) -> TrainValidationSplit: ...
    def setSeed(self, value: int) -> TrainValidationSplit: ...
    def setParallelism(self, value: int) -> TrainValidationSplit: ...
    def setCollectSubModels(self, value: bool) -> TrainValidationSplit: ...
    def copy(self, extra: Optional[ParamMap] = ...) -> TrainValidationSplit: ...
    def write(self) -> MLWriter: ...
    @classmethod
    def read(cls: Type[TrainValidationSplit]) -> MLReader: ...

class TrainValidationSplitModel(
    Model,
    _TrainValidationSplitParams,
    MLReadable[TrainValidationSplitModel],
    MLWritable,
):
    bestModel: Model
    validationMetrics: List[float]
    subModels: List[Model]
    def __init__(
        self,
        bestModel: Model,
        validationMetrics: List[float] = ...,
        subModels: Optional[List[Model]] = ...,
    ) -> None: ...
    def setEstimator(self, value: Estimator) -> TrainValidationSplitModel: ...
    def setEstimatorParamMaps(
        self, value: List[ParamMap]
    ) -> TrainValidationSplitModel: ...
    def setEvaluator(self, value: Evaluator) -> TrainValidationSplitModel: ...
    def copy(self, extra: Optional[ParamMap] = ...) -> TrainValidationSplitModel: ...
    def write(self) -> MLWriter: ...
    @classmethod
    def read(cls: Type[TrainValidationSplitModel]) -> MLReader: ...
